From: Mike Snitzer <snitzer@redhat.com>

DM no longer needlessly holds md->bdev->bd_inode->i_mutex when changing
the size of a DM device.  This additional locking is unnecessary because
i_size_write() is already protected by the existing critical section in
dm_swap_table().  DM already has a reference on md->bdev so the
associated bd_inode may be changed without lifetime concerns.

A negative side-effect of having held md->bdev->bd_inode->i_mutex was
that a concurrent DM device resize and flush (via fsync) would
deadlock.  Dropping md->bdev->bd_inode->i_mutex eliminates this
potential for deadlock.  The following reproducer no longer deadlocks:
https://www.redhat.com/archives/dm-devel/2009-July/msg00284.html

Signed-off-by: Mike Snitzer <snitzer@redhat.com>
Signed-off-by: Mikulas Patocka <mpatocka@redhat.com>
Cc: stable@kernel.org

---
 drivers/md/dm.c |    7 +++++--
 1 file changed, 5 insertions(+), 2 deletions(-)

Index: linux-2.6.37-rc4/drivers/md/dm.c
===================================================================
--- linux-2.6.37-rc4.orig/drivers/md/dm.c
+++ linux-2.6.37-rc4/drivers/md/dm.c
@@ -1996,9 +1996,12 @@ static void __set_size(struct mapped_dev
 {
 	set_capacity(md->disk, size);
 
-	mutex_lock(&md->bdev->bd_inode->i_mutex);
+	/*
+	 * Only DM is allowed to change the size of a DM device.
+	 * i_size_write() is protected by the dm_swap_table() critical
+	 * section that uses md->suspend_lock.
+	 */
 	i_size_write(md->bdev->bd_inode, (loff_t)size << SECTOR_SHIFT);
-	mutex_unlock(&md->bdev->bd_inode->i_mutex);
 }
 
 /*
